package ostate.invoke.impl;

import ostate.invoke.MethodInvoker;
import ostate.invoke.InvocationResult;
import ostate.invoke.MethodInvokingContext;
import ostate.invoke.DifferentResultsException;

import java.lang.reflect.InvocationTargetException;

/**
 * Two default invokers:
 * From children to root, and from root to children.
 * Throws {@link DifferentResultsException} if states returns
 * different not-null results. 
 *
 * @author Pavel Mavrin
 * @version $Id$
 */

public enum DefaultMethodInvoker implements MethodInvoker {

    ROOT_CHILDREN {
        public InvocationResult invoke(MethodInvokingContext context)
                throws IllegalAccessException, InvocationTargetException {
            InvocationResult result = context.processThisState();
            while (context.hasMoreChildren()) {
                InvocationResult newResult = context.processNextChild();
                result = mergeResults(result, newResult);
            }
            return result;
        }
    },

    CHILDREN_ROOT {
        public InvocationResult invoke(MethodInvokingContext context)
                throws IllegalAccessException, InvocationTargetException {
            InvocationResult result = InvocationResult.EMPTY;
            while (context.hasMoreChildren()) {
                InvocationResult newResult = context.processNextChild();
                result = mergeResults(result, newResult);
            }
            InvocationResult newResult = context.processThisState();
            result = mergeResults(result, newResult);
            return result;
        }
    };

    private static InvocationResult mergeResults(
            InvocationResult result1,
            InvocationResult result2) {
        if (result2.getResult() == null) {
            return result1;
        } else {
            if (result1.getResult() == null) {
                return result2;
            } else {
                if (!result1.getResult().equals(result2.getResult())) {
                    throw new DifferentResultsException(
                            result1.getReturnedBy(),
                            result2.getReturnedBy(),
                            result1.getResult(),
                            result2.getResult()
                    );
                }
                return result1;
            }
        }
    }
}
