package ostate.invoke.impl;

import ostate.util.tree.Node;
import ostate.invoke.InvocationResult;
import ostate.invoke.MethodInvoker;
import ostate.invoke.MethodInvokingContext;
import ostate.IState;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.Stack;

/**
 * {@link MethodInvokingContext} implementation
 *
 * @author Pavel Mavrin
 * @version $Id$
 */

public class MethodInvokingContextImpl<I> implements MethodInvokingContext {

    private final Node<IState<I>> node;
    private final Method method;
    private final Object[] args;
    private final MethodInvoker invoker;
    private final Iterator<Node<IState<I>>> childrenIterator;
    private final Stack<IState<I>> sources;

    public MethodInvokingContextImpl(Node<IState<I>> node, Method method,
                                     MethodInvoker invoker,
                                     Stack<IState<I>> sources,
                                     Object... args) {
        this.node = node;
        this.method = method;
        this.args = args;
        this.invoker = invoker;
        this.sources = sources;
        childrenIterator = node.getChildren().iterator();
    }

    public InvocationResult processThisState()
            throws IllegalAccessException, InvocationTargetException {
        sources.push(node.getValue());
        try {
            return new InvocationResult(
                    method.invoke(node.getValue(), args),
                    node.getValue());
        } finally {
            sources.pop();
        }
    }

    public InvocationResult processNextChild()
            throws IllegalAccessException, InvocationTargetException {
        return invoker.invoke(
                new MethodInvokingContextImpl<I>(
                        childrenIterator.next(),
                        method,
                        invoker,
                        sources,
                        args));
    }

    public boolean hasMoreChildren() {
        return childrenIterator.hasNext();
    }

}
