package ostate.util.tree;

import java.util.List;
import java.util.ArrayList;

/**
 * Node to be documented.
 *
 * @author Pavel Mavrin
 * @version $Id$
 */
public class Node<V>{

    private final List<Node<V>> children;

    private final Node<V> parent;

    private final Tree<V> tree;

    private V value;

    public Node(Tree<V> tree, Node<V> parent, V value) {
        this.parent = parent;
        this.value = value;
        this.tree = tree;
        tree.registerNode(this);
        children = new ArrayList<Node<V>>();
    }

    public void setValue(V value) {
        tree.disregisterNode(this);
        this.value = value;
        tree.registerNode(this);
    }

    public void addChild(Node<V> child) {
        children.add(child);
    }

    public void addChild(V value) {
        addChild(new Node<V>(tree, this, value));
    }

    public void removeAllChildren() {
        Node<V>[] children = new Node[this.children.size()];
        children = this.children.toArray(children);
        for (Node<V> child : children) {
            removeChild(child);
        }
    }

    public void removeChild(Node<V> child) {
        child.removeAllChildren();
        tree.disregisterNode(child);
        children.remove(child);
    }

    public Node<V> getParent() {
        return parent;
    }

    public List<Node<V>> getChildren() {
        return children;
    }

    public V getValue() {
        return value;
    }
}
