package connection.states;

import connection.Connection;
import connection.events.ConnectionError;
import ostate.IStateMachine;
import ostate.annotations.State;
import ostate.annotations.Handler;

/**
 * Connected to be documented.
 *
 * @author Pavel Mavrin
 * @version $Id$
 */
@State (
        name = "connected"
)
public class Connected extends AbstractState {

    public Connected(IStateMachine<Connection> sm) {
        super(sm);
    }

    public void stateEntry() {
        System.out.println("Connected state entry.");
    }

    public void stateExit() {
        System.out.println("Connected state exit.");
    }

    public void connect() {
        System.out.println("ConnectionError: already connected");
    }

    public void disconnect() {
        sm.changeState(Disconnected.class);
    }

    @Handler public void handleError(ConnectionError error) {
        super.handleError(error);
        System.out.println("Error??? Oops... Time to disconnect.");
        sm.changeState(Disconnected.class);
    }
}
